from blinker import signal

myst_yaml_frontmatter_register = signal("myst_yaml_frontmatter_register")
"""
Register custom YAML types for frontmatter processing
"""

myst_yaml_frontmatter_render = signal("myst_yaml_frontmatter_render")
"""
Render some Myst content from frontmatter 
"""



from __future__ import annotations

import collections

import yaml

from markdown_it import MarkdownIt

from .signals import myst_yaml_frontmatter_register, myst_yaml_frontmatter_render


STR_TAG = "tag:yaml.org,2002:str"
MYST_PREFIXES = "!md", "!myst"


class FrontmatterLoader(yaml.Loader):
    """
    Custom YAML Loader for Myst Frontmatter

    - Mapping order is respected (wiht OrderedDict)
    """
    def __init__(self, stream, myst: MarkdownIt):
        super().__init__(self, stream)
        for _, pair in myst_yaml_frontmatter_register.send(self):
            if not len(pair) == 2:
                log.warning(
                    "Ignoring YAML type (%s), expected a (tag, handler) tuple", pair
                )
                continue
            tag, constructor = pair
            self.add_constructor(tag, constructor)
        

    def construct_mapping(self, node, deep=False):
        """User OrderedDict as default for mappings"""
        return collections.OrderedDict(self.construct_pairs(node))


def render(content: str) -> str:
    results = myst_yaml_frontmatter_render.send("myst", content)
    if len(results) < 1:
        # TODO: warn missing rendered
        pass
    elif len(results) > 1:
        # TODO: warn ignored renderers
        pass
    rendered = results[0][1]
    return rendered.strip()

def myst_constructor(loader, node):
    """Allows to optionnaly parse Markdown in multiline literals"""
    value = loader.construct_scalar(node)
    return render(value)

def multiline_myst_constructor(loader, node):
    """Allows to optionnaly parse Markdown in multiline literals"""
    value = loader.construct_scalar(node)
    return render(value) if node.style == "|" else value


for prefix in MYST_PREFIXES:
    FrontmatterLoader.add_constructor(prefix, myst_constructor)


class LiteralFrontmatterLoader(FrontmatterLoader):
    """
    A custom YAML loader for Myst Frontmatter considering all literals as Myst content
    """

LiteralFrontmatterLoader.add_constructor(STR_TAG, multiline_myst_constructor)



@property
def loader_class(self):

    FrontmatterLoader.add_constructor("!md", self.yaml_markdown_constructor)
    if self.settings.get("MYST_FRONTMATTER_PARSE_LITERAL", True):
        FrontmatterLoader.add_constructor(
            STR_TAG, self.yaml_multiline_as_markdown_constructor
        )
    for _, pair in myst_yaml_register.send(self):
        if not len(pair) == 2:
            log.warning(
                "Ignoring YAML type (%s), expected a (tag, handler) tuple", pair
            )
            continue
        tag, constructor = pair
        FrontmatterLoader.add_constructor(tag, constructor)

    return FrontmatterLoader



import logging
import re

from functools import cached_property
from typing import Any, MutableMapping, Sequence

import yaml

from markdown_it import MarkdownIt
from markdown_it.renderer import RendererHTML
from markdown_it.token import Token
from markdown_it.utils import OptionsDict

# Core plugins, always enabled
from mdit_py_plugins.front_matter import front_matter_plugin
from mdit_py_plugins.colon_fence import colon_fence_plugin
from mdit_py_plugins.myst_role import myst_role_plugin
from mdit_py_plugins.myst_blocks import myst_block_plugin

from pygments import highlight
from pygments.formatters import HtmlFormatter
from pygments.lexers import TextLexer, get_lexer_by_name

DELIMITER = "---"
BOUNDARY = re.compile(r"^{0}$".format(DELIMITER), re.MULTILINE)
STR_TAG = "tag:yaml.org,2002:str"

INTERNAL_LINK = re.compile(r"^%7B(\w+)%7D")

log = logging.getLogger(__name__)

PLUGINS = dict(
    tasklist=tasklists_plugin,
)


class HtmlRenderer(RendererHTML):
    """
    An altered MarkdownIt HTML rendered taking reader settings in account.
    """
    def fence(
        self,
        tokens: Sequence[Token],
        idx: int,
        options: OptionsDict,
        env: MutableMapping,
    ) -> str:
        highlighted = super().fence(tokens, idx, options, env)
        if env.get("pygments"):
            return (
                "<div class=highlight>"
                + highlighted
                + "</div>\n"
            )
        return highlighted


class MystCodeFormatter(HtmlFormatter):
    def wrap(self, source, outfile):
        """
        Wrap the ``source``, which is a generator yielding
        individual lines, in custom generators. See docstring
        for `format`. Can be overridden.
        """
        return source


class MystIt(MarkdownIt):
    def __init__(self, **options):
        super().__init__("gfm-like", options_update=options, renderer_cls=HtmlRenderer)
        self.use(front_matter_plugin).use(colon_fence_plugin).use(myst_role_plugin).user(myst_block_plugin)

    # def normalizeLink(self, url: str) -> str:
    #     normalized = super().normalizeLink(url)
    #     return INTERNAL_LINK.sub(r"{\g<1>}", normalized)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mystit', 'mystit.plugins']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'blinker>=1.4,<2.0',
 'markdown-it-py[linkify,plugins]>=2.0.0,<3.0.0',
 'mdit-py-plugins>=0.3,<0.4']

extras_require = \
{'lint': ['black>=21.12b0,<22.0',
          'flake8<4.0',
          'isort>=5.10.1,<6.0.0',
          'mypy>=0.920,<0.921',
          'types-mock>=4.0.4,<5.0.0'],
 'pygments': ['Pygments>=2.10.0,<3.0.0'],
 'test': ['pyquery>=1.4.3,<2.0.0',
          'pytest>=6.2.5,<7.0.0',
          'pytest-cov>=3.0.0,<4.0.0',
          'pytest-pythonpath>=0.7.3,<0.8.0',
          'pytest-sugar>=0.9.4,<0.10.0']}

setup_kwargs = {
    'name': 'mystit',
    'version': '0.1.0',
    'description': 'A MarkdownIt-Py preconfigured for Myst syntax',
    'long_description': None,
    'author': 'Axel H.',
    'author_email': 'noirbizarre@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
